/**
* Filename: Event.java (in org.openbandy.event)
* This file is part of the BandyBasics project.
*
* www.openbandy.org
*/

package org.openbandy.event;

import java.util.Enumeration;
import java.util.Hashtable;

/**
 * This class implements a simple thread that dispatches a given
 * <code>Event</code> to all <code>EventSinks</code>. This way,
 * the thread running the EventRouter is not blocked during the
 * time it takes the EventSinks to absorb the events.
 * 
 * <br><br>
 * (c) Copyright Philipp Bolliger 2007, ALL RIGHTS RESERVED.
 * 
 * @author Philipp Bolliger (philipp@bolliger.name)
 * @version 1.0
 */
public class EventDispatcherThread extends Thread {
    
    /* The event to dispatch */
    private Event event;
    
    /* An enumeratio of event sinks to which the event
     * must be dispatched. */ 
    private Enumeration eventSinks;
    
    /* Hashtable of filters :  event sink -> filter */
    private Hashtable eventFilters;
    
    /* Constructor */
    public EventDispatcherThread(Event event, Enumeration eventSinks, Hashtable eventFilters){
        super("EventDispatcherThread");
        this.event = event;
        this.eventSinks = eventSinks;
        this.eventFilters = eventFilters;
    }

    public void run() {
        while(eventSinks.hasMoreElements()){
            EventSink sink = (EventSink)eventSinks.nextElement();
            if(sink != null){
                EventFilter filter = (EventFilter)eventFilters.get(sink);
                if(filter != null){
                    /* send event to filter */
                    filter.setEventSink(sink);
                    filter.filter(event);
                    
                    /* if provided, send event unfiltered to respective event sink */
                    EventSink eventSinkUnfiltered = filter.getEventSinkUnfiltered();
                    if(eventSinkUnfiltered != null){
                        eventSinkUnfiltered.absorb(event);
                    }
                }else{
                    sink.absorb(event);
                }
            }
        }
    }
    
    
    
    

}
