/**
 * Filename: MessageSerializationTest.java (in org.openbandy.example.test)
 * This file is part of the OpenBandy project.
 *
 * www.openbandy.org
 */

package org.openbandy.example.test;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;

import org.openbandy.example.pingpong.PingPongProtocol;
import org.openbandy.io.Message;
import org.openbandy.io.stream.StreamSerializer;
import org.openbandy.service.LogService;
import org.openbandy.service.SerializerService;
import org.openbandy.test.Test;

/**
 * TODO describe purpose and usage
 *
 * <br><br>
 * (c) Copyright Philipp Bolliger 2007, ALL RIGHTS RESERVED.
 *
 * @author Philipp Bolliger (philipp@bolliger.name)
 * @version 0.1
 * 
 */
public class MessageSerializationTest extends Test {

	/* Constructor */
	public MessageSerializationTest() {
		super("Message Serialization");
	}

	public void run() {
		testStarted();

		try {
			/* ***** Test the serialization of a message without content, COMPRESSION_NONE ***** */

			/* create a message with content == null */
			Message message = new Message();
			message.setSender("TestSender");
			message.setType(0);

			/* serialize the message into a stream */
			ByteArrayOutputStream baos = SerializerService.getXmlStream(message, StreamSerializer.COMPRESSION_NONE);

			/* deserialize the message object from the stream */
			ByteArrayInputStream bais = new ByteArrayInputStream(baos.toByteArray());
			Message clonedMessage = (Message)SerializerService.getObjectFromXmlStream(bais);

			/* compare deserialized message object with original */
			if(!clonedMessage.isEqual(message)) {
				LogService.warn(this, "Message serialization test failed (content==null, COMPRESSION_NONE)");
				testsScreen.setTestFailed(this);
				return;
			}
			else{
				testsScreen.setTestPassed(this);
			}


			/* ***** Test the serialization of a message without content, COMPRESSED_TOKEN ***** */

			/* serialize the message into a stream */
			baos = SerializerService.getXmlStream(message, StreamSerializer.COMPRESSED_TOKEN);

			/* deserialize the message object from the stream */
			bais = new ByteArrayInputStream(baos.toByteArray());
			clonedMessage = (Message)SerializerService.getObjectFromXmlStream(bais);

			/* compare deserialized message object with original */
			if(!clonedMessage.isEqual(message)) {
				LogService.warn(this, "Message serialization test failed (content==null, COMPRESSED_TOKEN)");
				testsScreen.setTestFailed(this);
				return;
			}
			else{
				testsScreen.setTestPassed(this);
			}


			/* ***** Test the serialization of a message without content, COMPRESSED_ZIP ***** */

			/* serialize the message into a stream */
			baos = SerializerService.getXmlStream(message, StreamSerializer.COMPRESSED_ZIP);

			/* deserialize the message object from the stream */
			bais = new ByteArrayInputStream(baos.toByteArray());
			clonedMessage = (Message)SerializerService.getObjectFromXmlStream(bais);

			/* compare deserialized message object with original */
			if(!clonedMessage.isEqual(message)) {
				LogService.warn(this, "Message serialization test failed (content==null, COMPRESSED_ZIP)");
				testsScreen.setTestFailed(this);
				return;
			}
			else{
				testsScreen.setTestPassed(this);
			}


			/* ***** Test the serialization of a message without content, COMPRESSED_BEST ***** */

			/* serialize the message into a stream */
			baos = SerializerService.getXmlStream(message, StreamSerializer.COMPRESSED_BEST);

			/* deserialize the message object from the stream */
			bais = new ByteArrayInputStream(baos.toByteArray());
			clonedMessage = (Message)SerializerService.getObjectFromXmlStream(bais);

			/* compare deserialized message object with original */
			if(!clonedMessage.isEqual(message)) {
				LogService.warn(this, "Message serialization test failed (content==null, COMPRESSED_BEST)");
				testsScreen.setTestFailed(this);
				return;
			}
			else{
				testsScreen.setTestPassed(this);
			}



			/* ***** Test the serialization of a message with content, COMPRESSION_NONE ***** */

			/* get a ping pong request message */
			PingPongProtocol protocol = new PingPongProtocol();
			message = protocol.createMessage("TestSender", "Hello World", 0);

			/* serialize the message into a stream */
			baos = SerializerService.getXmlStream(message, StreamSerializer.COMPRESSION_NONE);

			/* deserialize the message object from the stream */
			bais = new ByteArrayInputStream(baos.toByteArray());
			clonedMessage = (Message)SerializerService.getObjectFromXmlStream(bais);

			/* compare deserialized message object with original */
			if(!clonedMessage.isEqual(message)) {
				LogService.warn(this, "Message serialization test failed (content!=null, COMPRESSION_NONE)");
				testsScreen.setTestFailed(this);
				return;
			}
			else{
				testsScreen.setTestPassed(this);
			}


			/* ***** Test the serialization of a message with content, COMPRESSED_TOKEN ***** */

			/* serialize the message into a stream */
			baos = SerializerService.getXmlStream(message, StreamSerializer.COMPRESSED_TOKEN);

			/* deserialize the message object from the stream */
			bais = new ByteArrayInputStream(baos.toByteArray());
			clonedMessage = (Message)SerializerService.getObjectFromXmlStream(bais);

			/* compare deserialized message object with original */
			if(!clonedMessage.isEqual(message)) {
				LogService.warn(this, "Message serialization test failed(content!=null, COMPRESSED_TOKEN)");
				testsScreen.setTestFailed(this);
				return;
			}
			else{
				testsScreen.setTestPassed(this);
			}
			
			
			/* ***** Test the serialization of a message with content, COMPRESSED_ZIP ***** */

			/* serialize the message into a stream */
			baos = SerializerService.getXmlStream(message, StreamSerializer.COMPRESSED_ZIP);

			/* deserialize the message object from the stream */
			bais = new ByteArrayInputStream(baos.toByteArray());
			clonedMessage = (Message)SerializerService.getObjectFromXmlStream(bais);

			/* compare deserialized message object with original */
			if(!clonedMessage.isEqual(message)) {
				LogService.warn(this, "Message serialization test failed(content!=null, COMPRESSED_ZIP)");
				testsScreen.setTestFailed(this);
				return;
			}
			else{
				testsScreen.setTestPassed(this);
			}
			
			
			/* ***** Test the serialization of a message with content, COMPRESSED_BEST ***** */

			/* serialize the message into a stream */
			baos = SerializerService.getXmlStream(message, StreamSerializer.COMPRESSED_BEST);

			/* deserialize the message object from the stream */
			bais = new ByteArrayInputStream(baos.toByteArray());
			clonedMessage = (Message)SerializerService.getObjectFromXmlStream(bais);

			/* compare deserialized message object with original */
			if(!clonedMessage.isEqual(message)) {
				LogService.warn(this, "Message serialization test failed(content!=null, COMPRESSED_BEST)");
				testsScreen.setTestFailed(this);
				return;
			}
			else{
				testsScreen.setTestPassed(this);
			}
		}
		catch (Exception e) {
			LogService.error(this, e.getMessage(), e);
			testsScreen.setTestFailed(this);
		}

		testFinished();
	}

}
