/**
 * Filename: RmsSerializerTest.java (in org.openbandy.example.test)
 * This file is part of the BandySerializer project.
 *
 * www.BandySerializer.org
 */

package org.openbandy.example.test;

import org.openbandy.example.Car;
import org.openbandy.example.EuropeanSportscar;
import org.openbandy.io.rms.RmsStorage;
import org.openbandy.service.LogService;
import org.openbandy.service.SerializerService;
import org.openbandy.test.Test;

/**
 * TODO describe purpose and usage
 *
 * <br><br>
 * (c) Copyright Philipp Bolliger 2007, ALL RIGHTS RESERVED.
 *
 * @author Philipp Bolliger (philipp@bolliger.name)
 * @version 0.1
 * 
 */
public class RmsSerializerTest extends Test {


	public RmsSerializerTest() {
		super("Rms Serializer Test");
	}

	public void run() {
		try{
			testStarted();

			/* create a car and a sportscar object */
			Car car = TestDataGenerator.createDummyCar();
			EuropeanSportscar europeanSportscar = TestDataGenerator.createDummyEuropeanSportscar();

			/* store both objects in the rms */
			SerializerService.storeToRms(car);
			SerializerService.storeToRms(europeanSportscar);

			//TODO remove dev code
			RmsStorage.getInstance().printAllRecordStores();

			/* create a new car object instance and load values from the object just stored to the rms */
			Car clonedCar = new Car();
			SerializerService.loadFromRms(clonedCar, car.getIdValue());

			/* create a new european sports car object instance and load values from the object just stored to the rms */
			EuropeanSportscar clonedEuropeanSportscar = new EuropeanSportscar();
			SerializerService.loadFromRms(clonedEuropeanSportscar, europeanSportscar.getIdValue());

			/* delete the car object in the rms */
			if(!RmsStorage.getInstance().delete(car)){
				testsScreen.setTestFailed(this);
				return;
			}

			/* delete the european sports car object in the rms */
			if(!RmsStorage.getInstance().delete(europeanSportscar)){
				testsScreen.setTestFailed(this);
				return;
			}

			/* test deletion of car object */
			if(testDeletion(car)){

				/* compare original object with clone */
				if(!clonedCar.isEqual(car)) {
					LogService.warn(this, "Cloned car is not equal with original");
					testsScreen.setTestFailed(this);
					return;
				}
			}

			/* test deletion of european sports car object */
			if(testDeletion(europeanSportscar)){

				/* compare original object with clone */
				if(!clonedEuropeanSportscar.isEqual(europeanSportscar)) {
					LogService.warn(this, "Cloned europeanSportscar is not equal with original");
					testsScreen.setTestFailed(this);
					return;
				}
			}

			testsScreen.setTestPassed(this);
		}
		catch(Exception e){
			testsScreen.setTestFailed(this);
			LogService.error(this, "Rms serializer test failed", e);
		}

		testFinished();
	}

	public boolean testDeletion(Car car){
		/* check if the object is actually deleted from the rms */
		if (RmsStorage.getInstance().existsInStorage(car) != 0) {
			LogService.warn(this, "Could not delete "+car.getLegibleString()+" from rms");
			testsScreen.setTestFailed(this);
			return false;
		}
		else if (RmsStorage.getInstance().existsInStorage(car.getTires()) != 0) {
			LogService.warn(this, "Object ("+car.getTires().getLegibleString()+") not deleted from rms");
			testsScreen.setTestFailed(this);
			return false;
		}
		else if (RmsStorage.getInstance().existsInStorage(car.getEngine()) != 0) {
			LogService.warn(this, "Object ("+car.getEngine().getLegibleString()+") not deleted  from rms");
			testsScreen.setTestFailed(this);
			return false;
		}
		else if (RmsStorage.getInstance().existsInStorage(car.getEngine().getCrankshaft()) != 0) {
			LogService.warn(this, "Object ("+car.getEngine().getCrankshaft().getLegibleString()+") not deleted from rms");
			testsScreen.setTestFailed(this);
			return false;
		}
		return true;
	}

}
