/**
* Filename: RmsConnection.java (in org.openbandy.io.rms)
* This is part of the OpenBandy project.
* 
* www.openbandy.org
*/

package org.openbandy.io.rms;

import java.io.DataOutputStream;
import java.io.IOException;
import java.util.Enumeration;
import java.util.Vector;

import javax.microedition.rms.InvalidRecordIDException;
import javax.microedition.rms.RecordStore;
import javax.microedition.rms.RecordStoreException;
import javax.microedition.rms.RecordStoreFullException;
import javax.microedition.rms.RecordStoreNotFoundException;
import javax.microedition.rms.RecordStoreNotOpenException;

import org.openbandy.io.Serializable;
import org.openbandy.io.SerializationIdentifiers;
import org.openbandy.io.storage.ObjectEntry;
import org.openbandy.io.storage.StorageConnection;
import org.openbandy.service.LogService;

/**
* Wrapper class to access 1 RecordStore.
*
* <br><br>
* (c) Copyright P. Bolliger 2007, ALL RIGHTS RESERVED.
*
* @author Philipp Bolliger (philipp@bolliger.name)
* @version 1.0
*/
public class RmsConnection implements StorageConnection {
    
    private RecordStore recordStore;
    private String recordStoreName;
    
    private boolean hasHeader = false;
    
    private boolean isConnected = false;
    
    /* always use the same instances in order to save heap memory */
    DirectByteArrayOutputStream dbaos;
    DataOutputStream dos;
    
    protected RmsConnection(String recordStoreName){
        this.recordStoreName = recordStoreName;
        this.dbaos = new DirectByteArrayOutputStream();
        this.dos = new DataOutputStream(this.dbaos);
        
    }
    
    /**
     * TODO comment!
     *
     * @return
     */
    protected boolean hasHeader(){
        return hasHeader;
    }
    /**
     * TODO comment!
     *
     */
    protected void setHasHeader(){
    	this.hasHeader = true;
    }
    /**
     * TODO comment!
     *
     */
    protected void setupAsRecordStoreWithHeader(){
        this.hasHeader = true;
    	
        /* add header field */
        try{
        	getRecordStore().addRecord(null,0,0);
    	}catch(RecordStoreFullException rsfe){
    		LogService.error(this, rsfe.getMessage(), rsfe);
    	}catch(RecordStoreNotFoundException rsnfe){
    		LogService.error(this, rsnfe.getMessage(), rsnfe);
    	}catch(RecordStoreNotOpenException rsnoe){
    		LogService.error(this, rsnoe.getMessage(), rsnoe);
    	}catch(RecordStoreException rse){
    		LogService.error(this, rse.getMessage(), rse);
    	}
    }
    
    /**
     * TODO comment!
     *
     * @return
     */
    public RecordStore getRecordStore(){
        return recordStore;
    }
    
    /**
     * TODO comment!
     *
     * @return
     */
    public String getRecordStoreName(){
        return recordStoreName;
    }
    
    /* (non-Javadoc)
     * @see org.openbandy.io.storage.StorageConnection#connect()
     */
    public boolean connect() {
        if(! isConnected){
            try{
                recordStore = RecordStore.openRecordStore(recordStoreName, false);
                isConnected = true;
            }catch(RecordStoreNotFoundException rsnfe){
                LogService.error(this, "No object in RMS with name"+recordStoreName, rsnfe);
                return false;
            }catch(RecordStoreFullException rsfe){
                LogService.error(this, "RecordStore for "+recordStoreName+" is full : "+rsfe.getMessage(), rsfe);
                return false;
            }catch(RecordStoreException rse){
                LogService.error(this, "Could not connect to RecordStore : "+rse.getMessage(), rse);
                return false;
            }
        }
        return true;
    }
    
	/* (non-Javadoc)
	 * @see org.openbandy.io.storage.StorageConnection#isConnected()
	 */
	public boolean isConnected() {
		return isConnected;
	}
    
    /**
     * TODO comment!
     *
     * @return
     */
    protected boolean connectAndCreateIfNecessary(){
        try{
            recordStore = RecordStore.openRecordStore(recordStoreName, true);
            isConnected = true;
        }catch(RecordStoreFullException rsfe){
            LogService.error(this, "RecordStore for "+recordStoreName+" is full : "+rsfe.getMessage(), rsfe);
            return false;
        }catch(RecordStoreException rse){
            LogService.error(this, "Could not connect to RecordStore : "+rse.getMessage(), rse);
            return false;
        }
        return true;
    }
    
    /* (non-Javadoc)
     * @see org.openbandy.io.storage.StorageConnection#close()
     */
    public void close() {
    	if(recordStore != null){
            try{
                recordStore.closeRecordStore();
                isConnected = false;
            }catch(RecordStoreException rse){
                LogService.error(this, rse.getMessage(), rse);
            }
        }
    }
    
    /* (non-Javadoc)
     * @see org.openbandy.io.storage.StorageConnection#getConnectionIdentifier()
     */
    public String getConnectionIdentifier() {
        return recordStoreName;
    }
    
    /* (non-Javadoc)
     * @see org.openbandy.io.storage.StorageConnection#executeINSERT(org.openbandy.io.storage.ObjectEntry)
     */
    public int executeINSERT(ObjectEntry objectEntry) {
        if (isConnected){
            int id = 1;
            Serializable owner = objectEntry.getOwner();
            
            LogService.debug(this, "INSERT : "+RmsStorage.getRecordStoreName(owner));
            
            Vector objAttr = RmsStorage.getObjectAttributes(owner);
            Enumeration elel = objAttr.elements();
            while(elel.hasMoreElements()){
                String name = (String)elel.nextElement();
                String value = objectEntry.get(name);
                LogService.debug(this, "-value : "+value);
                updateStringAt(id++, value);
            }
            
            return owner.getIdValue();
        }else{
            LogService.warn(this, "Not connected");
        }
        return 0;
    }
    
    /* (non-Javadoc)
     * @see org.openbandy.io.storage.StorageConnection#executeUPDATE(org.openbandy.io.storage.ObjectEntry)
     */
    public void executeUPDATE(ObjectEntry objectEntry) {
        if (isConnected){
            int id = 1;
            Serializable owner = objectEntry.getOwner();
            
            LogService.debug(this, "UPDATE : "+RmsStorage.getRecordStoreName(owner));
            
            Vector objAttr = RmsStorage.getObjectAttributes(owner);
            Enumeration elel = objAttr.elements();
            while(elel.hasMoreElements()){
                String name = (String)elel.nextElement();
                String value = objectEntry.get(name);
                LogService.debug(this, "-value : "+value);
                updateStringAt(id++, value);
            }
        }else{
            LogService.warn(this, "Not connected");
        }
    }
    
    /* (non-Javadoc)
     * @see org.openbandy.io.storage.StorageConnection#executeDELETE(org.openbandy.io.Serializable)
     */
    public boolean executeDELETE(Serializable serializable) {
        /* assure that connection has control */
        String name = RmsStorage.getRecordStoreName(serializable);
        if (name.equals(recordStoreName)){
            if(isConnected){
                close();
            }
            deleteRecordStore(name);
            return true;
        }
        return false;
    }
    
    /**
     * TODO comment!
     *
     * @return
     */
    protected synchronized int getNumberOfRecords(){
        if(isConnected){
            try{
                return recordStore.getNumRecords();
            }catch(RecordStoreNotOpenException rsno){
                LogService.error(this, rsno.getMessage(), rsno);
            }
        }else{
            LogService.warn(this, "Not connected");
        }
        return 0;
    }
    
    /**
     * TODO comment!
     *
     * @param recordStoreName
     */
    protected static synchronized void deleteRecordStore(String recordStoreName){
        try{  
            RecordStore.deleteRecordStore(recordStoreName);
            LogService.debug(RmsConnection.class.getName(), "Deleted RecordStore : "+recordStoreName);
        }catch(RecordStoreNotFoundException rsnfe){
            LogService.error(RmsConnection.class.getName(), recordStoreName+" not found. "+rsnfe.getMessage(), rsnfe);
        }catch(RecordStoreException rse){
            LogService.error(RmsConnection.class.getName(), "Could not delete "+recordStoreName+". "+rse.getMessage(), rse);
        }
    }
    
    /**
     * Create RecordStore and create dummy entries.
     * 
     * @param recordStoreName
     * @param size
     */
    protected void createEmptyRecordStore(String recordStoreName, int size){
        try{
            recordStore = RecordStore.openRecordStore(recordStoreName, true);
            isConnected = true;
            for (int i=0; i<size; i++){
                recordStore.addRecord(null, 0, 0);
            }
        }catch(RecordStoreFullException rsfe){
            LogService.error(this, rsfe.getMessage(), rsfe);
        }catch(RecordStoreNotFoundException rsnfe){
            LogService.error(this, rsnfe.getMessage(), rsnfe);
        }catch(RecordStoreNotOpenException rsnoe){
            LogService.error(this, rsnoe.getMessage(), rsnoe);
        }catch(RecordStoreException rse){
            LogService.error(this, rse.getMessage(), rse);
        }
        
    }
    
    /**
     * TODO comment!
     *
     * @param recordId
     */
    protected synchronized void deleteRecord(int recordId){
        try{
            recordStore.deleteRecord(recordId);
        }catch(InvalidRecordIDException irie){
            LogService.error(this, "No object with id : "+recordId+" "+irie.getMessage(), irie);
        }catch(RecordStoreNotOpenException rsnoe){
            LogService.error(this, rsnoe.getMessage(), rsnoe);
        }catch(RecordStoreException rse){
            LogService.error(this, rse.getMessage(), rse);
        }
    }
    
    /**
     * TODO comment!
     *
     * @return
     */
    protected synchronized int getNewIdFromRecordStore(){
        if(isConnected){
            int newId = SerializationIdentifiers.NO_ID;
            try{
                newId = recordStore.addRecord(null, 0, 0);
            }catch(RecordStoreNotOpenException rsnoe){
                LogService.error(this, rsnoe.getMessage(), rsnoe);
            }catch(RecordStoreFullException rsfe){
                LogService.error(this, rsfe.getMessage(), rsfe);
            }catch(RecordStoreException rse){
                LogService.error(this, rse.getMessage(), rse);
            }
            return newId;
        }else{
            LogService.warn(this, "Not connected");
        }
        return 0;
    }
    
    /**
     * TODO comment!
     *
     * @param id
     * @param value
     */
    protected void updateStringAt(int id, String value){
        if(value == null){
            value = "";
        }
        if(isConnected){
            try{
                dos.writeUTF(value); dos.flush();
                byte[] data = dbaos.getByteArray();
                recordStore.setRecord(id, data, 0, dbaos.size());
                dbaos.reset();
            }catch(RecordStoreFullException rsfe){
                LogService.error(this, rsfe.getMessage(), rsfe);
            }catch(InvalidRecordIDException irie){
                LogService.error(this, irie.getMessage(), irie);
            }catch(RecordStoreNotOpenException rsne){
                LogService.error(this, rsne.getMessage(), rsne);
            }catch(RecordStoreException rse){
                LogService.error(this, rse.getMessage(), rse);
            }catch(IOException ioe){
                LogService.error(this, ioe.getMessage(), ioe);
            }
        }else{
            LogService.warn(this, "Not connected");
        }
    }
    
    /**
     * TODO comment!
     *
     * @param value
     * @return
     */
    protected int addString(String value){
        if(value == null){
            value = "";
        }
        if(isConnected){
            try{
                dos.writeUTF(value); dos.flush();
                byte[] data = dbaos.getByteArray();
                int size = dbaos.size();
                dbaos.reset();
                return recordStore.addRecord(data, 0, size);
            }catch(RecordStoreFullException rsfe){
                LogService.error(this, rsfe.getMessage(), rsfe);
            }catch(RecordStoreNotOpenException rsne){
                LogService.error(this, rsne.getMessage(), rsne);
            }catch(RecordStoreException rse){
                LogService.error(this, rse.getMessage(), rse);
            }catch(IOException ioe){
                LogService.error(this, ioe.getMessage(), ioe);
            }
            return 0;
        }else{
            LogService.warn(this, "Not connected");
        }
        return 0;
    }
    
    /**
     * TODO comment!
     *
     * @return
     */
    protected RmsTypeRecordStoreHeader getHeader(){
        if(hasHeader){
            RmsRecord record = new RmsRecord(recordStore);
            record.moveTo(1);
            String headerString = record.readUTF();
            
            RmsTypeRecordStoreHeader trsh = new RmsTypeRecordStoreHeader();
            trsh.setHeader(headerString);
            return trsh;
        }else{
            LogService.debug(this, "RecordStore is not set to be a type record store");
            return null;
        }
    }
    
    
    /* ************ Method for Developing Purposes only ************* */
    
    protected void printRecordStore(){
        try{
            RmsRecord record = new RmsRecord(recordStore);

            /* read the first Record */
            record.moveTo(1);
            String recordValue = record.readUTF();
            
            /* handle record stores with header */
            if(recordValue.startsWith(RmsTypeRecordStoreHeader.ID_DELIMITER) && 
                    recordValue.endsWith(RmsTypeRecordStoreHeader.ID_DELIMITER)){
                
            	/* get header */
                RmsTypeRecordStoreHeader trsh = new RmsTypeRecordStoreHeader();
                trsh.setHeader(recordValue);
                
                /* print header */
                if(recordStoreName.equals(RmsStorage.TYPES_RECORDSTORE_NAME)){
            		System.out.println("  (Types Record Store : header = '"+recordValue+"', size : "+(recordStore.getNumRecords()-1)+")");
                }else{
                    System.out.println("  (Type Store : header = '"+recordValue+"', size : "+(recordStore.getNumRecords()-1)+")");	
                }
                
                /* print out all entries according to ids in header field */
                Enumeration elements = trsh.getTypeRecordStoreIds();
                while(elements.hasMoreElements()){
                    String idString = (String)elements.nextElement();
                    int id = Integer.parseInt(idString);
                    
                    record.moveTo(id);
                    /* print value */
                    System.out.println("id: "+id+"  value : "+record.readUTF());
                }
            }
            /* handle object stores */
            else{
            	System.out.println("  (Object Store)");
            	for(int i=1;i<=recordStore.getNumRecords(); i++){
            		record.moveTo(i);
            		System.out.println("value : "+record.readUTF());
            	}
            }
        }catch(Exception e){}
    }
    
}
