/**
* Filename: RmsRecord.java (in org.openbandy.io.rms)
* This is part of the OpenBandy project.
* 
* www.openbandy.org
*/

package org.openbandy.io.rms;

import java.io.ByteArrayInputStream;
import java.io.DataInput;
import java.io.DataInputStream;
import java.io.IOException;

import javax.microedition.rms.InvalidRecordIDException;
import javax.microedition.rms.RecordStore;
import javax.microedition.rms.RecordStoreException;
import javax.microedition.rms.RecordStoreNotOpenException;

import org.openbandy.service.LogService;


/**
 * In order to have a more efficient way to read records all access
 * is encapsulated in this class, hence garanteing that:
 * 	1. reuse the raw byte array
 * 	2. reuse the input streams
 * 
 * TODO add Javadoc and state exactly which values are returned
 * 			 in case of IOException
*
* <br><br>
* (c) Copyright P. Bolliger 2007, ALL RIGHTS RESERVED.
*
* @author Philipp Bolliger (philipp@bolliger.name)
* @version 1.0
*/
public class RmsRecord implements DataInput {

    private RecordStore rs;
    private byte[] data;
    private int length;
    private DataInputStream din;

    public RmsRecord( RecordStore rs ){
        this( rs, 100 );
    }

    public RmsRecord(RecordStore rs, int initialRecordSize ){
        this.rs = rs;
        this.data = new byte[ initialRecordSize ];
        this.din = new DataInputStream(new DirectByteArrayInputStream( data ) );
        this.length = -1;
    }


    /**
     * TODO comment!
     *
     * @return
     */
    public byte[] getByteArray(){
        return data;
    }


    /**
     * TODO comment!
     *
     * @return
     */
    public int getLength(){
        return length;
    }


    /**
     * TODO comment!
     *
     * @param id
     * @return
     */
    public byte[] moveTo( int id ){
        try{
	        length = rs.getRecordSize( id );
	
	        if( length > data.length ){
	            data = new byte[ length + 40 ];
	            din  = new DataInputStream( 
	                   new ByteArrayInputStream( data ) );
	        }
	
	        rs.getRecord( id, data, 0 );
	        din.reset();
	        
        }catch(InvalidRecordIDException irie){
            LogService.error(this, "No record with id:"+id, irie);
        }catch(RecordStoreNotOpenException rsnoe){
        	LogService.error(this, rsnoe.getMessage(), rsnoe);
        }catch(RecordStoreException rse){
        	LogService.error(this, rse.getMessage(), rse);
        }catch(IOException ioe){
        	LogService.error(this, ioe.getMessage(), ioe);
        }
        
        return data;
    }

    /* (non-Javadoc)
     * @see java.io.DataInput#readFully(byte[])
     */
    public void readFully(byte b[]){
        try{
            din.readFully( b );
        }catch(IOException ioe){
        	LogService.error(this, ioe.getMessage(), ioe);
        }   
    }


    /* (non-Javadoc)
     * @see java.io.DataInput#readFully(byte[], int, int)
     */
    public void readFully(byte b[], int off, int len){
		try{
		    din.readFully( b, off, len );
		}catch(IOException ioe){
			LogService.error(this, ioe.getMessage(), ioe);
		}
    }


    /* (non-Javadoc)
     * @see java.io.DataInput#readBoolean()
     */
    public boolean readBoolean() throws IOException{
        return din.readBoolean();
    }
 

    /* (non-Javadoc)
     * @see java.io.DataInput#readByte()
     */
    public byte readByte(){
        try{
            return din.readByte();
        }catch(IOException ioe){
        	LogService.error(this, ioe.getMessage(), ioe);
        }
        return 0;
    }

    /* (non-Javadoc)
     * @see java.io.DataInput#readUnsignedByte()
     */
    public int readUnsignedByte(){
        try{
            return din.readUnsignedByte();
        }catch(IOException ioe){
        	LogService.error(this, ioe.getMessage(), ioe);
        }
        return 0;
    }

    /* (non-Javadoc)
     * @see java.io.DataInput#readShort()
     */
    public short readShort(){
        try{
            return din.readShort();
        }catch(IOException ioe){
        	LogService.error(this, ioe.getMessage(), ioe);
        }
        return 0;
    }

    /* (non-Javadoc)
     * @see java.io.DataInput#readUnsignedShort()
     */
    public int readUnsignedShort(){
        try{
            return din.readUnsignedShort();
        }catch(IOException ioe){
        	LogService.error(this, ioe.getMessage(), ioe);
        }
        return 0;
    }

    /* (non-Javadoc)
     * @see java.io.DataInput#readChar()
     */
    public char readChar(){
        try{
            return din.readChar();
        }catch(IOException ioe){
        	LogService.error(this, ioe.getMessage(), ioe);
        }
        return 0;
    }
    
    /* (non-Javadoc)
     * @see java.io.DataInput#readInt()
     */
    public int readInt(){
        try{
            return din.readInt();
        }catch(IOException ioe){
        	LogService.error(this, ioe.getMessage(), ioe);
        }
        return 0;
    }

    /* (non-Javadoc)
     * @see java.io.DataInput#readLong()
     */
    public long readLong(){
        try{
            return din.readLong();
        }catch(IOException ioe){
        	LogService.error(this, ioe.getMessage(), ioe);
        }
        return 0;
    }

    /* (non-Javadoc)
     * @see java.io.DataInput#readUTF()
     */
    public String readUTF(){
        try{
            return din.readUTF();
        }catch(IOException ioe){
        	LogService.error(this, ioe.getMessage(), ioe);
        }
        return "";
    }

    /* (non-Javadoc)
     * @see java.io.DataInput#skipBytes(int)
     */
    public int skipBytes(int arg0){
        try{    
            return din.skipBytes(arg0);
        }catch(IOException ioe){
        	LogService.error(this, ioe.getMessage(), ioe);
        }
        return 0;
    }

    /* (non-Javadoc)
     * @see java.io.DataInput#readFloat()
     */
    public float readFloat(){
        try{    
            return din.readFloat();
        }catch(IOException ioe){
        	LogService.error(this, ioe.getMessage(), ioe);
        }
        return 0;
    }

    /* (non-Javadoc)
     * @see java.io.DataInput#readDouble()
     */
    public double readDouble(){
        try{    
            return din.readDouble();
        }catch(IOException ioe){
        	LogService.error(this, ioe.getMessage(), ioe);
        }
        return 0;
    }

    /**
     * TODO comment!
     *
     * @return
     * @throws IOException
     */
    public String readLine() throws IOException {
        return readUTF();
    }
}
