/**
* Filename: Log.java (in org.openbandy.log)
* This file is part of the BandyBasics project.
*
* www.BandyBasics.org
*/

package org.openbandy.log;


/**
 * The Log class provides methods to print log messages of different
 * severity to a canvas.
 *
 * <br><br>
 * (c) Copyright Philipp Bolliger 2007, ALL RIGHTS RESERVED.
 *
 * @author Philipp Bolliger (philipp@bolliger.name)
 * @version 1.0
 * 
 */
public interface Log {
	
	/**
	 * Logs a message with log level LEVEL_ERROR
	 * 
	 * @param origin The object reporting
	 * @param message	 The message to log
	 * @param exception The exception that was caused with this error
	 */
	public void error(Object origin, String message, Throwable exception);

	/**
	 * Logs a message with log level LEVEL_ERROR and prints the
	 * stack trace to System.out
	 * 
	 * @param originName The class name of the reporting object
	 * @param message The message to log
	 * @param exception The exception that was caused with this error
	 */
	public void error(String originName, String message, Throwable exception);

	/**
	 * Logs a message with log level LEVEL_WARNING
	 * 
	 * @param origin The object reporting
	 * @param message	 The message to log
	 */
	public void warn(Object origin, String message);
	
	/**
	 * Logs a message with log level LEVEL_WARNING
	 * 
	 * @param origin The class name of the object reporting
	 * @param message	 The message to log
	 */
	public void warn(String originName, String message);

	/**
	 * Logs a message with log level LEVEL_INFO
	 * 
	 * @param origin The object reporting
	 * @param message	 The message to log
	 */
	public void info(Object origin, String message);
	
	/**
	 * Logs a message with log level LEVEL_INFO
	 * 
	 * @param origin The class name of the object reporting
	 * @param message	 The message to log
	 */
	public void info(String originName, String message);

	/**
	 * Logs a message with log level LEVEL_DEBUG
	 * 
	 * @param origin The object reporting
	 * @param message	 The message to log
	 */
	public void debug(Object origin, String message);
	
	/**
	 * Logs a message with log level LEVEL_DEBUG
	 * 
	 * @param origin The class name of the object reporting
	 * @param message	 The message to log
	 */
	public void debug(String originName, String message);
	
	/**
	 * Set the log level
	 *
	 * @param level Log level according to @link org.openbandy.log.LogLevel
	 */
	public void setLogLevel(int level);
}
