package org.openbandy.log;

import java.io.IOException;

import javax.microedition.lcdui.Choice;
import javax.microedition.lcdui.Command;
import javax.microedition.lcdui.CommandListener;
import javax.microedition.lcdui.Display;
import javax.microedition.lcdui.Displayable;
import javax.microedition.lcdui.Image;
import javax.microedition.lcdui.List;

import org.openbandy.service.LogService;
import org.openbandy.ui.BandyDisplayable;



/**
 * This Form is be used to set the level of filtering
 * (i.e. which kind of log messages are shown in the
 * log canvas).
 * 
 * @version 1.0
 * @author Philipp Bolliger (philipp@bolliger.name)
 */
public class SelectLogModeForm extends List implements BandyDisplayable, CommandListener{

	/* Commands */
	private Command cmdSelect;
	private Command cmdOk;
	private Command cmdCancel;

	/* Images */
	private Image imgChecked;
	private Image imgUnchecked;
	
	/** Reference to the MIDlets display */
	private Display display;
	
	/** Reference to the log canvas */
	private LogImpl logCanvas;

	/* ************ Constructor ***************** */

	public SelectLogModeForm() {
		super("Log Mode", Choice.IMPLICIT);

		/* compose the GUI */
		cmdSelect = new Command("Select", Command.SCREEN, 1);
		cmdOk = new Command("Ok", Command.OK, 1);
		cmdCancel = new Command("Cancel", Command.CANCEL, 1);
		addCommand(cmdSelect);
		addCommand(cmdOk);
		addCommand(cmdCancel);
		
		/* Select is the default command */
		this.setSelectCommand(cmdSelect);

		/* load images */
		try{
			imgChecked = Image.createImage("/img/checked.png");
			imgUnchecked = Image.createImage("/img/unchecked.png");
		}catch(IOException ioe){
			LogService.error(this, ioe.getMessage(), ioe);
		}
		
		/* make myself command listener */
		this.setCommandListener(this);
	}

	
	/* ******************** Methods for BandyDisplayable ************** */
	
	public void setDisplay(Display display, Displayable previousDisplayable) {
		this.display = display;
		try {
			this.logCanvas = (LogImpl)previousDisplayable;
			
			/* append levels, set images correctly */
			for(int i = 0;i<logCanvas.config.modes.length;i++){
				if(logCanvas.config.modes[i]){
					append(LogConfiguration.modeLabels[i], imgChecked);
				}else{
					append(LogConfiguration.modeLabels[i], imgUnchecked);
				}
			}

		}
		catch(ClassCastException cce){
			LogService.error(this, "Must have LogCanvas as previous screen", cce);
		}
	}


	public void updateScreen() {
		/* is updated by START_INQUIRY and SEARCH_SERVICES threads */
	}
	
	
	/* ******************** Methods for CommandListener ************** */
	
	public void commandAction(Command c, Displayable d) {
		if(c == cmdSelect) {
			try {
				int index = getSelectedIndex();
				logCanvas.config.setLogMode(index, !logCanvas.config.modes[index]);
				if(logCanvas.config.modes[index]){
					set(index, LogConfiguration.modeLabels[index], imgChecked);
				}
				else{
					set(index, LogConfiguration.modeLabels[index], imgUnchecked);
				}
			} catch (Exception e) {
				LogService.error(this, e.getMessage(), e);
			}
		}
		else if(c == cmdCancel){
			display.setCurrent(logCanvas);
		}else if(c == cmdOk){
			logCanvas.resetCanvas();
			display.setCurrent(logCanvas);
		}
	}
}
